/*
 * --------------------------------------------
 * RBC::GUI::WebDialog:
 *   Wrapper to communicate with UI::WebDialog.
 *
 *
 * Author : BC8 & Sugar
 * Version: 1.2
 * Date   : 2017.6.23
 *
 * --------------------------------------------
 * WebDialogX:
 *   https://sketchucation.com/forums/viewtopic.php?f=180&t=46626&p=417591&hilit=WebDialogX#p417591
 *
 * --------------------------------------------
*/

/*
* **************************
* ERROR
* **************************
* */

// Capture global JS errors and output to the console of SketchUp.
window.onerror= function(message, source, lineno, colno, error){
    var data = {
        "message":message.toString(),
        "source":source.toString(),
        "lineno":lineno,
        "colno":colno.toString(),
        "error":error.toString()
    };
    window.location = 'skp:_error@'+JSON.stringify(data);
};

/*
* **************************
* console.log
* **************************
* */

//
function log_info(message) {
    if( window.console != undefined ){
        console.log(message);
    }
}

/*
* **************************
* Language
* **************************
* */

/*
* Checking and displaying browser compatibility.
* **Browser below IE8 is not supported.**
*
* */
function checkBrowser() {
    if(navigator.appName == 'Microsoft Internet Explorer'){
        var version = navigator.appVersion .split(";")[1].replace(/[ ]/g,"").replace('MSIE',"");
        if(parseFloat(version) < 8.0){
            var warnDiv = document.createElement('textarea');
            var style = warnDiv.style;
            style.height= '30px';
            style.width= '100%';
            style.background= '#363636';
            style.color= '#00868B';
            style.fontSize= '20px';
            warnDiv.value= '(Please upgrade your browser to IE8 or more!)浏览器升级到IE8以上!';
            document.body.insertBefore(warnDiv, document.body.childNodes[0]);
            if( window.console != undefined ){
                console.log('');
            }
        }
    }
    return true;
}

/*
* **************************
* WebDialog
* **************************
* */

var RBC = RBC || {};

//
RBC.Language = {
    //
    strings:{},

    // Initialization language value
    refresh: function(callbackFun) {
        log_info("refresh RBC.Language.");
        if(/SketchUp/i.test(navigator.userAgent)){
            if(typeof(callbackFun)=="function"){
                this._refreshFun= callbackFun;
            }
            window.location= 'skp:_language@language';
        }else {
            this.strings = {};
            callbackFun.call(RBC.Language)
        }
        return null;
    },

    // Get language value
    tr: function(key) {
        if(!/SketchUp/i.test(navigator.userAgent)){
            log_info("RBC.Language: " + key + "<==>"+key);
            return key;
        }
        if(this.strings[key]){
            log_info("RBC.Language: " + key + "<==>"+this.strings[key]);
            return this.strings[key];
        }else{
            window.location= 'skp:_language_insert_key@'+key;
            return key;
        }
    },

    //
    _refreshFun:function () {}

};

RBC.GUI = RBC.GUI || {};

RBC.GUI.WebDialog = {


  // close this dialog.
  close: function () {
      if (/SketchUp/i.test(navigator.userAgent)) {
          window.location = 'skp:close@';
          return true
      }else{
          if( window.console != undefined ){
            console.log('The dialog can only be closed in SketchUp.');
          }
        return false
      }

  },

  /* Wrapper to call any defined action_callback, handles escaping and encoding etc.
   * { name: string, data: json, test: function, success: function, error: function, complete: function}
   * name, *arguments
   */
  callback: function () {
    // merge the given arguments with
    var opt = {
      data    : null,
      direct  : false,
      indirect: true,
      success : function () {}, // null,
      error   : function () {}, // null,
      complete: function () {}  // null
    },
      name = null,
      args = [];
    if(arguments.length == 1 && typeof(arguments[0]) === 'object'){
      json= arguments[0];
      for(var key in json){
        opt[key]= json[key]
      }
      name = opt.name;
      args.push(opt.data);
      //
      if(!/SketchUp/i.test(navigator.userAgent)){
        if(typeof opt.test !== 'undefined'){
            value= opt.test(args);
            opt.complete(value);
            return true;
        }else{
            if( window.console != undefined ){
                console.log('Error(J): Please add the Test function key!');
            }
            return false;
        }
      }
    } else if(arguments.length >= 1) {
        name = arguments[0];
        args = Array.prototype.slice.call(arguments).slice(1);
        //
        if (!/SketchUp/i.test(navigator.userAgent)) {
            if( window.console != undefined ){
                console.log('Error(J): Please execute it in SU!');
            }
            return false
        }
    }else {
        if( window.console != undefined ){
            console.log('Error(J): Incorrect parameter!');
        }
        return false
    }
    var id = (this._messageIdCounter++);
    this._callbacks[id] = {
      success: opt.success,
      error: opt.error,
      complete: opt.complete
    };

    /* direct message passing
     *   encode message into url
     *   Ruby gets the message immediately
     */
    if (opt.direct == true ||  opt.indirect == false){
      var data = JSON.stringify( args );
      data = encodeURIComponent(data.replace(/%/g, "%25").replace(/'/g, "%q"));
      // split the argument string into chunks and encode each
      var chunks = data.match(/.{1,2060}(?=%|$)/g); // actually 2083 is maximum url size in Internet Explorer
      for(var i=0; i<chunks.length; i++){
        this._setLocation("skp:_collect_data@[" + id + "]" + chunks[i]);
      }
      this._setLocation("skp:" + name + "@" + this._id_label + id);
    }
    /* indirect message passing (preferred)
     *   put message into hidden text field
     *   tell Ruby about it
     *   Ruby gets the text field value
     */
    else if(opt.indirect == true || opt.direct == false){
      this._createMessageField(id, JSON.stringify(args));
      this._setLocation("skp:" + name + "@" + this._id_label +  id);
    }
    return true; // the message has been sent
  },


  /* Call a Ruby method (even if a specific action_callback has not been defined)
   * Arguments: name and any amount of arguments  or  options object
   */
  callTo: function () {
    if(arguments.length == 1 && typeof(arguments[0]) === 'object'){
      var opt = arguments[0];
      opt.data = [opt.name, opt.data];
      opt.name = "_callTo";
      this.callback(opt);
    } else if(arguments.length >= 1) {
      var name_and_args = Array.prototype.slice.call(arguments).slice();
      name_and_args.unshift("_callTo");
      this.callback.apply(this, name_and_args);
    } else {
      return false
    }
  },


  /* Execute any Ruby code (even if a specific action_callback has not been defined)
   */
  execScript: function (code_string, options) {
    if(typeof(code_string) !== 'string'){ return false }
    if(typeof(options) === 'object'){
      options.data = code_string;
      options.name = "_execScript";
      return this.callback(options);
    } else {
      return this.callback("_execScript", code_string);
    }
  },

  // Keeps track of callbacks.
  _messageIdCounter: 0,
  _callbacks: {},
  _id_label : '__<GUID>__',

  _createMessageField: function (id, value) {
    var messageField = document.createElement('input');
    messageField.setAttribute('type', 'hidden');
    messageField.setAttribute('style', 'display:none');
    messageField.setAttribute('id', 'RBC.GUI.WebDialog.' + id);
    messageField.value = value;
    document.body.appendChild(messageField);
  },
  _setLocation: function (url) {
    // allow the DOM to refresh so that the MessageField is created (from DynamicComponents dcBridge.js)
    window.setTimeout(function() {
      window.location.href = url;
    }, 0);
  },
  _cleanUp: function (id) {
    try {
      var elem = document.getElementById('RBC.GUI.WebDialog.' + id);
      elem.parentNode.removeChild(elem);
    } catch(e) {
      //alert("Error(J): RBC.GUI.WebDialog._cleanUp:\\n" + e);
      window.location = 'skp:_error@'+ "Error(J): RBC.GUI.WebDialog._cleanUp:\\n" + e;
    } finally {
      delete this._callbacks[id];
    }
  },
  _cleanUpScripts: function () {
    // timeout to wait after injected script has been executed (otherwise this one is not removed)
    window.setTimeout(function(){
      var scripts = document.body.getElementsByTagName("script");
      for(var i = 0; i < scripts.length; i++){ scripts[i].parentNode.removeChild(scripts[i]) };
    }, 0);
  },
  _fromRuby: function (id, functionName, argumentsString) {
    var returnValue = null;
    try{
      //Create the function
      var fn = eval(functionName); // alternative only for global functions: window[functionName]
      if(fn == null){ throw "function is not defined" };
      //argumentsString = argumentsString;
      var data = eval(argumentsString);
      returnValue = fn.apply(fn, data);
    } catch(e){
      //alert("Error(J): RBC.GUI.WebDialog._fromRuby:\\n" + e);
      window.location = 'skp:_error@'+"Error(J): RBC.GUI.WebDialog._fromRuby:\\n" + e
    } finally {
      this._createMessageField(id, JSON.stringify(returnValue));
    }
  },
  _execScript: function (data) {
    var returnValue = null;
    try{
      returnValue = eval(data);
    } catch(e){
      //alert("Error(J): RBC.GUI.WebDialog._execScript:\\n" + e);
        window.location = 'skp:_error@'+"Error(J): RBC.GUI.WebDialog._execScript:\\n" + e
    }
    return returnValue;
  }
};
//RBC.GUI.WebDialog.init_language();